#include <QApplication>
#include <QWidget>
#include <QVBoxLayout>
#include <QGraphicsScene>
#include <QGraphicsItem>
#include <QGraphicsView>

#include <QScreen>
#include <QWindow>
#include <QDebug>

#include <algorithm>

bool optFullViewportUpdate = false;

static QString title()
{
    return QLatin1String(qVersion()) + QLatin1Char(' ')
            + QGuiApplication::platformName();
}

template <class Stream>
static void formatRect(Stream &d, const QRect &r)
{
    d << r.width() << 'x' << r.height()
     #if QT_VERSION >= 0x050E00
      << Qt::forcesign
     #else
      << forcesign
     #endif
      << r.x() << r.y()
     #if QT_VERSION >= 0x050E00
      << Qt::noforcesign
     #else
      << noforcesign
     #endif
         ;
}

template <class Stream>
static void dumpScreen(Stream &d, const QScreen *s)
{
    d << '"' << s->name() << "\" " << ' ' << s->logicalDotsPerInch()
      << "DPI , dpr=" << s->devicePixelRatio() << ' ';
    formatRect(d,  s->geometry());
}

class Widget : public QWidget
{
public:
    Widget(QWidget *parent = nullptr);

    QGraphicsView * view;
    QGraphicsScene * scene;
    QGraphicsItem * rectangle;
    QGraphicsItem * rectangle2;
};

Widget::Widget(QWidget *parent) : QWidget(parent)
{
    auto vl=  new QVBoxLayout(this);

    scene = new QGraphicsScene();
    view = new QGraphicsView(this);
    vl->addWidget(view);

    view->fitInView(this->rect());
    view->setScene(scene);
    // Adding FullViewportUpdate helps on High dpi screens
    if (optFullViewportUpdate)
        view->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    qDebug() << view->viewportUpdateMode();
    scene->setSceneRect(0,0,this->width(),this->height());

    QBrush greenBrush(Qt::green);
    QBrush blueBrush(Qt::blue);
    QPen outlinePen(Qt::black);
    outlinePen.setWidth(2);

    rectangle = scene->addRect(100, 0, 80, 100, outlinePen, blueBrush);
    rectangle2 = scene->addRect(200, 60, 200, 300, outlinePen, greenBrush);
    rectangle->setFlag(QGraphicsItem::ItemIsMovable);
    rectangle2->setFlag(QGraphicsItem::ItemIsMovable);
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

    if (args.contains("-n")) {
        qDebug("AA_DisableHighDpiScaling");
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    } else {
        qDebug("AA_EnableHighDpiScaling");
        QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    }

#if QT_VERSION >= 0x050E00
    if (args.contains("-p")) {
        qDebug("PassThrough");
        QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
    }
#endif

    QApplication a(argc, argv);

    QString t = title();
    {
        QDebug d = qDebug().noquote().nospace();
        d << t << ' ' << args << '\n';
#if QT_VERSION >= 0x050E00
        d << ", roundingPolicy=" << QGuiApplication::highDpiScaleFactorRoundingPolicy()<< '\n';
#endif // Qt 5.14
        for (auto s : QGuiApplication::screens()) {
            dumpScreen(d, s);
            d << '\n';
        }
    }

    optFullViewportUpdate = args.contains("-f");

    Widget Mw;
    Mw.setWindowTitle(t);
    Mw.show();

    return QCoreApplication::exec();
}
